#include <p24Fxxxx.h>
#include "TLS2130.h"
#include "lcdLib.h"

/*******************************************************************
* FUNCTION:     lcdPutUInt() 
* INPUTS:       num = unsigned integer number to display
*               radix = base to use when displaying number
* OUTPUTS:      None
* DESCRIPTION:  Writes a number to LCD at current cursor position
* AUTHOR:       Rob Ostapiuk
* DATE:         24 JAN 2008
* REVISIONS:    24 JAN 2008 - Initial Version
*******************************************************************/
void lcdPutUInt(unsigned long num, char radix)
{
    int i;                  // Loop index variable    
    /*--------------------------------------------------------------
      digits[] - Array to hold individual digits.
      NOTE: Digits stored in reverse order: LSD is in digits[0]
    --------------------------------------------------------------*/
    unsigned char digits[MAX_DIGITS];    
    /*--------------------------------------------------------------
      oldDigits[] - Array to hold previous value of digits[].
      Used to know which digits need to be updated on the display.
    --------------------------------------------------------------*/
    static unsigned char oldDigits[MAX_DIGITS];

    /*--------------------------------------------------------------
      (0) If the number is zero, just display a zero and exit.
    --------------------------------------------------------------*/    
    if (num == 0L)
    {
        lcdPutChar('0');
        return;
    }
    
    /*--------------------------------------------------------------
      (1) Initialize oldDigits[] so that a digit
      change may be detected on the first pass.
    --------------------------------------------------------------*/    
    for (i=0; i<MAX_DIGITS; i++)
        oldDigits[i] = 0;    
    
    /*--------------------------------------------------------------
      (2) If radix is hex, display a 0x prefix.
    --------------------------------------------------------------*/    
    if (radix == HEX) lcdPutStr("0x");
        
    /*--------------------------------------------------------------
      (3) Break up number into its individual digits and clear any
      unused elements of the digits array
    --------------------------------------------------------------*/
    for (i=0; i<MAX_DIGITS; i++)
    {
        digits[i] = 0;
        if (num == 0) continue;
        digits[i] = (char)(num % radix);
        num /= radix;
    }
    
    /*--------------------------------------------------------------
      (4) Write the digits to the LCD...
          Setup loop index to end of array and work back to front.
    --------------------------------------------------------------*/
    i = MAX_DIGITS-1;
    
    /*--------------------------------------------------------------
      (5) Skip over leading zeros.
    --------------------------------------------------------------*/
    while (digits[i] == 0) i--;
    
    /*--------------------------------------------------------------
      (6) For each remaining digit...
    --------------------------------------------------------------*/
    while (i >= 0)
    {
        /*----------------------------------------------------------
          (7) If the digit has changed since last written...
          NOTE: CANNOT PROPERLY HANDLE DOWN COUNT (trailing
          zeros not cleared from display)
        ----------------------------------------------------------*/
        if ((digits[i] != oldDigits[i]) || (digits[i] == 0))
        {
            /*------------------------------------------------------
              (8) If the digit has a value of 0 through 9, write
              its ASCII value to display ('0' = ASCII 48 decimal).
              This works for radix = BIN, OCT or DEC.
            ------------------------------------------------------*/
            if (digits[i] <= 9)
                lcdPutChar(digits[i]+48);
                
            /*------------------------------------------------------
              (9) If the digit has a value of 10 through 15, write
              the ASCII value for the letters A through F to the 
              display ('A' = ASCII 55 decimal).
              This is only used for radix = HEX.
            ------------------------------------------------------*/            
            else
                lcdPutChar(digits[i]+55);
        }
        /*----------------------------------------------------------
          (10) If the digit hasn't changed since last written,
          move the cursor one position to the right.
        ----------------------------------------------------------*/
        else
        {
            lcdCurFwd();  
        }              
        /*----------------------------------------------------------
          (11) Move to the next digit in digits[].
        ----------------------------------------------------------*/
        i--;
    }
    
    /*--------------------------------------------------------------
      (12) Copy current digits[] to oldDigits[]
    --------------------------------------------------------------*/
    for(i=0; i<MAX_DIGITS; i++)
    {
        oldDigits[i] = digits[i];
    }
}
